/*
 *  geo-platform
 *  Rich webgis framework
 *  http://geo-platform.org
 * ====================================================================
 *
 * Copyright (C) 2008-2013 geoSDI Group (CNR IMAA - Potenza - ITALY).
 *
 * This program is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version. This program is distributed in the 
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR 
 * A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details. You should have received a copy of the GNU General 
 * Public License along with this program. If not, see http://www.gnu.org/licenses/ 
 *
 * ====================================================================
 *
 * Linking this library statically or dynamically with other modules is 
 * making a combined work based on this library. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole combination. 
 * 
 * As a special exception, the copyright holders of this library give you permission 
 * to link this library with independent modules to produce an executable, regardless 
 * of the license terms of these independent modules, and to copy and distribute 
 * the resulting executable under terms of your choice, provided that you also meet, 
 * for each linked independent module, the terms and conditions of the license of 
 * that module. An independent module is a module which is not derived from or 
 * based on this library. If you modify this library, you may extend this exception 
 * to your version of the library, but you are not obligated to do so. If you do not 
 * wish to do so, delete this exception statement from your version. 
 *
 */
package org.geosdi.geoplatform.gui.server.gwt;

import com.google.gwt.user.client.rpc.SerializationException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import org.geosdi.geoplatform.gui.global.IGeoPlatformGlobal;
import org.geosdi.geoplatform.gui.server.service.IStartupService;
import org.geosdi.geoplatform.gui.server.spring.GPAutoInjectingRemoteServiceServlet;
import org.geosdi.geoplatform.gui.service.GeoPlatformConfiguration;
import org.geosdi.geoplatform.gui.spring.GeoPlatformContextUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

/**
 *
 * @author Giuseppe La Scaleia - CNR IMAA geoSDI Group
 * @email giuseppe.lascaleia@geosdi.org
 */
public class GeoPlatformConfigurationImpl extends GPAutoInjectingRemoteServiceServlet
        implements GeoPlatformConfiguration {

    private static final long serialVersionUID = 4416552134318747534L;
    static ThreadLocal<HttpServletRequest> perThreadRequest =
            new ThreadLocal<HttpServletRequest>();

    @Override
    public String processCall(String payload) throws SerializationException {
        try {
            perThreadRequest.set(getThreadLocalRequest());
            return super.processCall(payload);
        } finally {
            perThreadRequest.set(null);
        }
    }

    public static HttpServletRequest getRequest() {
        return perThreadRequest.get();
    }
    //
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    //
    @Autowired
    private IStartupService startupService;

    @Override
    public void init(ServletConfig config) throws ServletException {
        super.init(config);

        ApplicationContext context = WebApplicationContextUtils.getWebApplicationContext(
                getServletContext());
        GeoPlatformContextUtil.getInstance().setSpringContext(context);

        this.initGeoPlatformContext();
    }

    /**
     * (non-Javadoc)
     *
     * @see
     * org.geosdi.geoplatform.gui.service.GeoPlatformConfiguration#initGeoPlatformConfiguration()
     */
    @Override
    public IGeoPlatformGlobal initGeoPlatformConfiguration() {
        return startupService.initGeoPlatformConfiguration();
    }

    @Override
    public IGeoPlatformGlobal initGPConfigurationForBeanName(String beanName) {
        return startupService.initGPConfigurationForBeanName(beanName);
    }

    /**
     * Init GeoPlatform Context
     */
    private void initGeoPlatformContext() {
        assert (this.startupService != null) : "The GeoPlatform StartupService is null";

        logger.info("################ GeoPlatform Context Initialized Correctly");
    }
}
