/*
 *  geo-platform
 *  Rich webgis framework
 *  http://geo-platform.org
 * ====================================================================
 *
 * Copyright (C) 2008-2013 geoSDI Group (CNR IMAA - Potenza - ITALY).
 *
 * This program is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version. This program is distributed in the 
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR 
 * A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details. You should have received a copy of the GNU General 
 * Public License along with this program. If not, see http://www.gnu.org/licenses/ 
 *
 * ====================================================================
 *
 * Linking this library statically or dynamically with other modules is 
 * making a combined work based on this library. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole combination. 
 * 
 * As a special exception, the copyright holders of this library give you permission 
 * to link this library with independent modules to produce an executable, regardless 
 * of the license terms of these independent modules, and to copy and distribute 
 * the resulting executable under terms of your choice, provided that you also meet, 
 * for each linked independent module, the terms and conditions of the license of 
 * that module. An independent module is a module which is not derived from or 
 * based on this library. If you modify this library, you may extend this exception 
 * to your version of the library, but you are not obligated to do so. If you do not 
 * wish to do so, delete this exception statement from your version. 
 *
 */
package org.geosdi.geoplatform.gui.client.model.memento.save;

import java.util.ListIterator;
import org.geosdi.geoplatform.gui.action.ISave;
import org.geosdi.geoplatform.gui.client.config.MementoModuleInjector;
import org.geosdi.geoplatform.gui.client.model.memento.save.bean.MementoSaveAddedFolder;
import org.geosdi.geoplatform.gui.client.model.memento.save.bean.MementoSaveAddedLayers;
import org.geosdi.geoplatform.gui.model.memento.IMemento;
import org.geosdi.geoplatform.gui.model.tree.AbstractFolderTreeNode;
import org.geosdi.geoplatform.gui.model.tree.GPBeanTreeModel;
import org.geosdi.geoplatform.gui.model.tree.GPLayerTreeModel;

/**
 * @author Nazzareno Sileno - CNR IMAA geoSDI Group
 * @email nazzareno.sileno@geosdi.org
 *
 * This class is usefull on sync application version to delete the add operation
 * from cache after a remove operation that interest the same object
 */
public class MementoSaveCacheUtility {

    /**
     *
     * @return true if a creational memento was removed
     */
    public static boolean cleanCacheFromSaveAddOperation(GPBeanTreeModel element) {
        boolean condition = false;
        IMementoSave mementoSave = MementoModuleInjector.MainInjector.getInstance().getMementoSave();
        IMemento<ISave> memento;
        for (ListIterator<IMemento<ISave>> it = mementoSave.listIterator(); it.hasNext();) {
            memento = it.next();
            if (verifyCondition(element, memento)) {
                mementoSave.remove(memento);
                condition = true;
                break;
            }
        }
        return condition;
    }

    private static boolean verifyCondition(GPBeanTreeModel element, IMemento<ISave> memento) {
        boolean condition = false;
        if (element instanceof AbstractFolderTreeNode) {
            condition = verifyFolderCondition((AbstractFolderTreeNode) element, memento);
        } else if (element instanceof GPLayerTreeModel) {
            condition = verifyLayerCondition((GPLayerTreeModel) element, memento);
        }
        return condition;
    }

    private static boolean verifyLayerCondition(GPLayerTreeModel layer, IMemento<ISave> memento) {
        boolean condition = false;
        if (memento instanceof MementoSaveAddedLayers
                && ((MementoSaveAddedLayers) memento).getAddedLayers().size() == 1
                && ((MementoSaveAddedLayers) memento).getAddedLayers().get(0).getRefBaseElement().equals(layer)) {
            condition = true;
        }
        return condition;
    }

    private static boolean verifyFolderCondition(AbstractFolderTreeNode folder, IMemento<ISave> memento) {
        boolean condition = false;
        if (memento instanceof MementoSaveAddedFolder
                && ((MementoSaveAddedFolder) memento).getAddedFolder().getRefBaseElement().equals(folder)) {
            condition = true;
        }
        return condition;
    }
}
