/*
 *  geo-platform
 *  Rich webgis framework
 *  http://geo-platform.org
 * ====================================================================
 *
 * Copyright (C) 2008-2013 geoSDI Group (CNR IMAA - Potenza - ITALY).
 *
 * This program is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version. This program is distributed in the 
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR 
 * A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details. You should have received a copy of the GNU General 
 * Public License along with this program. If not, see http://www.gnu.org/licenses/ 
 *
 * ====================================================================
 *
 * Linking this library statically or dynamically with other modules is 
 * making a combined work based on this library. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole combination. 
 * 
 * As a special exception, the copyright holders of this library give you permission 
 * to link this library with independent modules to produce an executable, regardless 
 * of the license terms of these independent modules, and to copy and distribute 
 * the resulting executable under terms of your choice, provided that you also meet, 
 * for each linked independent module, the terms and conditions of the license of 
 * that module. An independent module is a module which is not derived from or 
 * based on this library. If you modify this library, you may extend this exception 
 * to your version of the library, but you are not obligated to do so. If you do not 
 * wish to do so, delete this exception statement from your version. 
 *
 */
package org.geosdi.geoplatform.gui.client.model;

import com.extjs.gxt.ui.client.GXT;
import com.google.gwt.user.client.ui.AbstractImagePrototype;
import java.util.List;
import org.geosdi.geoplatform.gui.client.LayerResources;
import org.geosdi.geoplatform.gui.client.model.visitor.VisitorModelConverter;
import org.geosdi.geoplatform.gui.configuration.map.client.layer.GPFolderClientInfo;
import org.geosdi.geoplatform.gui.configuration.map.client.layer.IGPFolderElements;
import org.geosdi.geoplatform.gui.model.tree.AbstractFolderTreeNode;
import org.geosdi.geoplatform.gui.model.tree.GPBeanTreeModel;
import org.geosdi.geoplatform.gui.model.tree.IGPNode;
import org.geosdi.geoplatform.gui.model.tree.TreeStatusEnum;

/**
 * @author Giuseppe La Scaleia - CNR IMAA geoSDI Group
 * @email giuseppe.lascaleia@geosdi.org
 *
 */
public class FolderTreeNode extends AbstractFolderTreeNode implements IGPNode {

    private static final long serialVersionUID = -3687415822526940729L;
    //
    private VisitorModelConverter visitor = new VisitorModelConverter(this);
    private boolean loading = Boolean.FALSE;
    private boolean expanded = Boolean.FALSE;

    public FolderTreeNode() {
    }

    public FolderTreeNode(String label) {
        super.setLabel(label);
    }

    public FolderTreeNode(GPFolderClientInfo folder, boolean addChildrens) {
        super(folder);
        super.numberOfDescendants = folder.getNumberOfDescendants();
        this.expanded = folder.isExpanded();
        if (addChildrens) {
            this.modelConverter(folder.getFolderElements());
        }
    }

    /**
     *
     * @param layersClientInfo
     */
    public final void modelConverter(List<IGPFolderElements> layersClientInfo) {
        for (IGPFolderElements element : layersClientInfo) {
            element.accept(this.visitor);
        }
    }

    /**
     * (non-Javadoc)
     *
     * @see org.geosdi.geoplatform.gui.model.tree.GPBeanTreeModel#getIcon()
     */
    @Override
    public AbstractImagePrototype getIcon() {
        if (this.isLoading()) {
            return GXT.IMAGES.icon_wait();
        }
        return LayerResources.ICONS.layerFolder();
    }

    public boolean isLoading() {
        return loading;
    }

    public void setLoading(boolean loading) {
        this.loading = loading;
    }

    @Override
    public boolean isExpanded() {
        return expanded;
    }

    public void setExpanded(boolean expanded) {
        this.expanded = expanded;
    }

    @Override
    public TreeStatusEnum getTreeStatus() {
        return TreeStatusEnum.FOLDER_SELECTED;
    }
}
