/*
 *  geo-platform
 *  Rich webgis framework
 *  http://geo-platform.org
 * ====================================================================
 *
 * Copyright (C) 2008-2013 geoSDI Group (CNR IMAA - Potenza - ITALY).
 *
 * This program is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version. This program is distributed in the 
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR 
 * A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details. You should have received a copy of the GNU General 
 * Public License along with this program. If not, see http://www.gnu.org/licenses/ 
 *
 * ====================================================================
 *
 * Linking this library statically or dynamically with other modules is 
 * making a combined work based on this library. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole combination. 
 * 
 * As a special exception, the copyright holders of this library give you permission 
 * to link this library with independent modules to produce an executable, regardless 
 * of the license terms of these independent modules, and to copy and distribute 
 * the resulting executable under terms of your choice, provided that you also meet, 
 * for each linked independent module, the terms and conditions of the license of 
 * that module. An independent module is a module which is not derived from or 
 * based on this library. If you modify this library, you may extend this exception 
 * to your version of the library, but you are not obligated to do so. If you do not 
 * wish to do so, delete this exception statement from your version. 
 *
 */
package org.geosdi.geoplatform.gui.client.widget;

import com.extjs.gxt.ui.client.event.SelectionChangedEvent;
import com.extjs.gxt.ui.client.widget.tips.ToolTipConfig;
import org.geosdi.geoplatform.gui.client.i18n.RoutingModuleConstants;
import org.geosdi.geoplatform.gui.client.model.GeocodingBean;
import org.geosdi.geoplatform.gui.client.mvc.RoutingController;
import org.geosdi.geoplatform.gui.puregwt.routing.RoutingHandlerManager;
import org.geosdi.geoplatform.gui.puregwt.routing.event.FinalRoutingPointEvent;
import org.geosdi.geoplatform.gui.puregwt.routing.event.RemoveFinalRoutingPointEvent;

/**
 * @author Giuseppe La Scaleia - CNR IMAA geoSDI Group
 * @email giuseppe.lascaleia@geosdi.org
 *
 */
public class FinalPointSearchWidget extends StartPointSearchRouting {

    private FinalRoutingPointEvent event;

    /**
     * @param controller
     */
    public FinalPointSearchWidget(RoutingController controller) {
        super(controller);
        this.event = new FinalRoutingPointEvent();
    }

    /**
     * (non-Javadoc)
     *
     * @see
     * org.geosdi.geoplatform.gui.client.widget.StartPointSearchRouting#setComboToolTip()
     */
    @Override
    public void setComboToolTip() {
        ToolTipConfig config = new ToolTipConfig();
        config.setTitle(RoutingModuleConstants.INSTANCE.usageText());
        config.setText(RoutingModuleConstants.INSTANCE.usageInstructionsText());
        config.setTrackMouse(true);
        this.combo.setToolTip(config);
    }

    /**
     * (non-Javadoc)
     *
     * @see
     * org.geosdi.geoplatform.gui.client.widget.StartPointSearchRouting#selectionChanged(com.extjs.gxt.ui.client.event.SelectionChangedEvent)
     */
    @Override
    public void changeSelection(SelectionChangedEvent<GeocodingBean> se) {
        event.setLocation(se.getSelectedItem());
        RoutingHandlerManager.fireEvent(event);
    }

    /**
     * (non-Javadoc)
     *
     * @see
     * org.geosdi.geoplatform.gui.client.widget.StartPointSearchRouting#clearGeoPlatformMap()
     */
    @Override
    public void cleanGeoPlatformMap() {
        RoutingHandlerManager.fireEvent(new RemoveFinalRoutingPointEvent());
    }
}
