/*
 *  geo-platform
 *  Rich webgis framework
 *  http://geo-platform.org
 * ====================================================================
 *
 * Copyright (C) 2008-2013 geoSDI Group (CNR IMAA - Potenza - ITALY).
 *
 * This program is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version. This program is distributed in the 
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR 
 * A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details. You should have received a copy of the GNU General 
 * Public License along with this program. If not, see http://www.gnu.org/licenses/ 
 *
 * ====================================================================
 *
 * Linking this library statically or dynamically with other modules is 
 * making a combined work based on this library. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole combination. 
 * 
 * As a special exception, the copyright holders of this library give you permission 
 * to link this library with independent modules to produce an executable, regardless 
 * of the license terms of these independent modules, and to copy and distribute 
 * the resulting executable under terms of your choice, provided that you also meet, 
 * for each linked independent module, the terms and conditions of the license of 
 * that module. An independent module is a module which is not derived from or 
 * based on this library. If you modify this library, you may extend this exception 
 * to your version of the library, but you are not obligated to do so. If you do not 
 * wish to do so, delete this exception statement from your version. 
 *
 */
package org.geosdi.geoplatform.connector.server.request;

import java.math.BigInteger;
import org.geosdi.geoplatform.connector.server.GPServerConnector;
import org.geosdi.geoplatform.gui.responce.CatalogFinderBean;
import org.geosdi.geoplatform.xml.csw.ConstraintLanguage;
import org.geosdi.geoplatform.xml.csw.ConstraintLanguageVersion;
import org.geosdi.geoplatform.xml.csw.OutputSchema;
import org.geosdi.geoplatform.xml.csw.TypeName;

/**
 * Abstract class of GetRecords CSW request
 * 
 * @author Giuseppe La Scaleia - CNR IMAA geoSDI Group
 * @email giuseppe.lascaleia@geosdi.org
 * @author Vincenzo Monteverde <vincenzo.monteverde@geosdi.org>
 */
public abstract class CatalogGetRecords<T> extends CatalogCSWRequest<T>
        implements CatalogGetRecordsRequest<T> {

    protected ConstraintLanguage constraintLanguage;
    protected ConstraintLanguageVersion constraintLanguageVersion;
    protected String constraint;
    protected CatalogFinderBean catalogFinder;
    protected BigInteger maxRecords;
    protected BigInteger startPosition;
    protected OutputSchema outputSchema;
    protected String resultType;
    protected String elementSetName;
    protected TypeName typeName;

    public CatalogGetRecords(GPServerConnector server) {
        super(server);
    }

    @Override
    public ConstraintLanguage getConstraintLanguage() {
        return constraintLanguage;
    }

    @Override
    public void setConstraintLanguage(ConstraintLanguage constraintLanguage) {
        this.constraintLanguage = constraintLanguage;
    }

    @Override
    public ConstraintLanguageVersion getConstraintLanguageVersion() {
        return constraintLanguageVersion;
    }

    @Override
    public void setConstraintLanguageVersion(ConstraintLanguageVersion constraintLanguageVersion) {
        this.constraintLanguageVersion = constraintLanguageVersion;
    }

    @Override
    public String getConstraint() {
        return constraint;
    }

    @Override
    public void setConstraint(String constraint) {
        this.constraint = constraint;
    }

    @Override
    public CatalogFinderBean getCatalogFinder() {
        return catalogFinder;
    }

    @Override
    public void setCatalogFinder(CatalogFinderBean catalogFinder) {
        this.catalogFinder = catalogFinder;
    }

    @Override
    public BigInteger getMaxRecords() {
        return maxRecords;
    }

    @Override
    public void setMaxRecords(BigInteger maxRecords) {
        this.maxRecords = maxRecords;
    }

    @Override
    public BigInteger getStartPosition() {
        return startPosition;
    }

    @Override
    public void setStartPosition(BigInteger startPosition) {
        this.startPosition = startPosition;
    }

    @Override
    public OutputSchema getOutputSchema() {
        return outputSchema;
    }

    @Override
    public void setOutputSchema(OutputSchema outputSchema) {
        // TODO GMD list
        if (outputSchema != OutputSchema.CSW_V202) {
            throw new UnsupportedOperationException("GMD output schema is not supported yet. "
                    + "Use CSW output schema.");
        }
        this.outputSchema = outputSchema;
    }

    @Override
    public String getResultType() {
        return resultType;
    }

    @Override
    public void setResultType(String resultType) {
        this.resultType = resultType;
    }

    @Override
    public String getElementSetName() {
        return elementSetName;
    }

    @Override
    public void setElementSetName(String elementSetName) {
        this.elementSetName = elementSetName;
    }

    @Override
    public TypeName getTypeName() {
        return typeName;
    }

    @Override
    public void setTypeName(TypeName typeName) {
        this.typeName = typeName;
    }

    @Override
    public String toString() {
        StringBuilder str = new StringBuilder(this.getClass().getSimpleName()).append(" {");
        str.append("constraintLanguage").append(constraintLanguage);
        str.append(", constraintLanguageVersion").append(constraintLanguageVersion);
        str.append(", constraint").append(constraint);
        str.append(", catalogFinder").append(catalogFinder);
        str.append(", maxRecords=").append(maxRecords);
        str.append(", startPosition=").append(startPosition);
        str.append(", outputSchema=").append(outputSchema);
        str.append(", resultType=").append(resultType);
        str.append(", elementSetName=").append(elementSetName);
        str.append(", typeName=").append(typeName);
        return str.append("}").toString();
    }
}
