/*
 *  geo-platform
 *  Rich webgis framework
 *  http://geo-platform.org
 * ====================================================================
 *
 * Copyright (C) 2008-2013 geoSDI Group (CNR IMAA - Potenza - ITALY).
 *
 * This program is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version. This program is distributed in the 
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR 
 * A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details. You should have received a copy of the GNU General 
 * Public License along with this program. If not, see http://www.gnu.org/licenses/ 
 *
 * ====================================================================
 *
 * Linking this library statically or dynamically with other modules is 
 * making a combined work based on this library. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole combination. 
 * 
 * As a special exception, the copyright holders of this library give you permission 
 * to link this library with independent modules to produce an executable, regardless 
 * of the license terms of these independent modules, and to copy and distribute 
 * the resulting executable under terms of your choice, provided that you also meet, 
 * for each linked independent module, the terms and conditions of the license of 
 * that module. An independent module is a module which is not derived from or 
 * based on this library. If you modify this library, you may extend this exception 
 * to your version of the library, but you are not obligated to do so. If you do not 
 * wish to do so, delete this exception statement from your version. 
 *
 */
package org.geosdi.geoplatform.connector.wfs;

import java.util.Arrays;
import javax.xml.namespace.QName;
import org.geosdi.geoplatform.connector.server.request.WFSGetFeatureRequest;
import org.geosdi.geoplatform.connector.server.request.WFSTransactionRequest;
import org.geosdi.geoplatform.connector.wfs.responce.AttributeDTO;
import org.geosdi.geoplatform.connector.wfs.responce.GeometryAttributeDTO;
import org.geosdi.geoplatform.gui.shared.wfs.TransactionOperation;
import org.geosdi.geoplatform.xml.wfs.v110.FeatureCollectionType;
import org.geosdi.geoplatform.xml.wfs.v110.ResultTypeType;
import org.geosdi.geoplatform.xml.wfs.v110.TransactionResponseType;
import org.junit.Test;

/**
 *
 * @author Vincenzo Monteverde <vincenzo.monteverde@geosdi.org>
 *
 * @author Giuseppe La Scaleia - CNR IMAA geoSDI Group
 * @email giuseppe.lascaleia@geosdi.org
 */
public class WFSTransactionUpdateTest extends WFSTestConfigurator {

    private final static QName POLY_LANDMARKS = new QName(
            "http://www.census.gov", "tiger:poly_landmarks");
    //
    private final static QName FIRESAT = new QName(
            "http://www.openplans.org/spearfish", "sf:firesat");

    @Test
    public void polyLandmarks() throws Exception {
        WFSTransactionRequest<TransactionResponseType> request =
                super.serverConnector.createTransactionRequest();

        request.setOperation(TransactionOperation.UPDATE);

        QName name = new QName(POLY_LANDMARKS.getLocalPart());
        request.setTypeName(name);

        request.setFID("poly_landmarks.1");

        AttributeDTO att = new AttributeDTO();
        att.setMaxOccurs(1);
        att.setMinOccurs(0);
        att.setName("LANAME");
        att.setNillable(true);
        att.setType("string");
        att.setValue("Washington Square Park.");
        request.setAttributes(Arrays.asList(att));

        // We must avoid transactions on the server,
        // but show only the requests generated

        logger.info("\n\nHERE THE REQUEST ########################## \n {}",
                request.showRequestAsString());

//        TransactionResponseType response = request.getResponse();
//        logger.info("\n*** {}", response.getTransactionResults());
//
//        TransactionSummaryType transactionSummary = response.getTransactionSummary();
//        Assert.assertEquals(0, transactionSummary.getTotalDeleted().intValue());
//        Assert.assertEquals(0, transactionSummary.getTotalInserted().intValue());
//        Assert.assertEquals(1, transactionSummary.getTotalUpdated().intValue());
//        Assert.assertEquals("1.1.0", response.getVersion());

        // HERE THE CODE TO TEST THE TRANSACTION RESULT WITH A SPECIFIC GET_FEATURE

//        WFSGetFeatureRequest<FeatureCollectionType> requestGet =
//                super.serverConnector.createGetFeatureRequest();
//
//        requestGet.setResultType(ResultTypeType.RESULTS.value());
//        requestGet.setTypeName(POLY_LANDMARKS);
//
//        requestGet.setFeatureIDs(Arrays.asList("poly_landmarks.1"));
//
//        logger.info("\n\n\nRESPONSE @@@@@@@@@@@@@@@@@@@@@@@@@@@@@ {}",
//                    requestGet.getResponseAsString());
    }

    @Test
    public void toppStatesTransaction() throws Exception {
        WFSTransactionRequest<TransactionResponseType> request =
                super.serverConnector.createTransactionRequest();

        request.setOperation(TransactionOperation.UPDATE);
        request.setTypeName(statesName);

        request.setFID("states.15");

        AttributeDTO attr = new AttributeDTO();
        attr.setName("STATE_ABBR");
        attr.setValue("TX");

        AttributeDTO attr1 = new AttributeDTO();
        attr1.setName("STATE_NAME");
        attr1.setValue("Texas");

        GeometryAttributeDTO geometry = new GeometryAttributeDTO();
        geometry.setName("the_geom");
        geometry.setSrid(new Integer(4326));

        geometry.setValue(
                "MULTIPOLYGON(((-104.02825076334733 35.19149566106441,-104.1009653599567 35.37565124147472,-103.99577238699258 35.64438803918993,-104.05615531619767 35.77197448783781,-104.08004995974001 35.837655548720065,-104.05754047393786 35.85581720990979,-104.03280926999736 35.905327614124765,-104.0094437423832 35.91142674582099,-103.95832005626964 35.9485079579948,-103.96312670883206 35.9642104872414,-103.92721094032994 35.96190322345904,-103.76124951117153 35.814990348640066,-103.50613092236235 35.590656945241996,-102.77095500522077 34.94078594734973,-102.7236284101646 34.89691823108883,-102.15615178527172 34.394360373634676,-102.13164780892022 34.3703315785713,-101.96066659396004 34.21677743301196,-101.69328437906898 33.97277012515911,-101.50939094779999 33.80321595690007,-101.43544697228172 33.85697754527128,-101.06406507410256 34.14718137853404,-100.67395412227033 34.43371204905564,-100.29722062968229 34.71409725385405,-100.12857680610038 34.83753625591273,-99.89483898015493 35.008950934791734,-99.46497324188395 35.330453006001555,-99.06985450704221 35.620765045695364,-98.87502859918 35.770219059027774,-98.68478839856576 35.9140753734212,-98.27960712785044 36.2132275186044,-98.17241678310354 36.293651880819716,-97.88868681708882 36.50678007349817,-97.49375780607525 36.80380519400589,-97.47278604075954 36.78666586391755,-96.91087590929142 36.29187689291593,-96.81954315658405 36.21575139434602,-96.53980495999313 35.96816607305017,-96.18368354403084 35.64768863178037,-96.09222617218897 35.569086669849355,-95.81610529267289 35.323861280737404,-95.44422503554703 34.99941247819986,-95.44158742504194 34.99500891829995,-95.83549587199415 34.6910179266243,-95.99370383132413 34.57105253223495,-96.23526188248803 34.389126965141166,-96.40686456562419 34.251441905662105,-96.62711292933908 34.085799522354165,-96.76220459910026 33.98080858172782,-97.01845230961601 33.78688616274638,-97.18176442264782 33.65786136799678,-97.16590540752978 33.64279468239158,-97.1315485500551 33.63816808099171,-97.12326134665992 33.62995966140495,-97.11608841259805 33.57709017679429,-97.12887798532589 33.54440626633634,-97.12313560284167 33.51393318255937,-97.13923992671714 33.44192624409633,-97.13563243366902 33.33883955241755,-97.08696633223045 33.28119857162395,-97.06144668063278 33.28242515147493,-97.03501942562616 33.2944749714538,-97.01317362929105 33.28399228425292,-96.98800449719624 33.244377179751275,-96.99091968187902 33.216168708400396,-96.9798406892659 33.177796203879936,-96.95665333878729 33.163268077145126,-96.92132316612478 33.17259443474667,-96.89166276763301 33.19403534875499,-96.85338326710047 33.191020676511876,-96.85905859259708 33.13981437113321,-96.8338599363424 33.09512506254092,-96.8517843911071 33.06624061492492,-96.85107496121178 33.011088814695974,-96.86876320102961 32.98681203202265,-96.91840081102355 32.95773288352005,-96.93720626277604 32.92691885309108,-96.93727635242617 32.91042482295206,-96.91460089881066 32.872737280511714,-96.87490727684542 32.84678473096404,-96.85399890466535 32.813434038267,-96.81814258252541 32.796529136641986,-96.80179554983263 32.75906118459092,-96.79031154656752 32.69768290940809,-96.74910915755942 32.6374557551164,-96.73946342971061 32.607532397051266,-96.69091599879636 32.561085142074425,-96.65814207841623 32.56009516058954,-96.6468776625022 32.54528320319383,-96.61253059938439 32.530946218998054,-96.60607329572949 32.514386566352535,-96.59352423533089 32.488556791554934,-96.61330575114133 32.451464808187765,-96.60848839334979 32.388240550226094,-96.58437064956985 32.34785657287244,-96.55600444421577 32.33906059534514,-96.5316192774987 32.3417472595869,-96.51640937594249 32.334967763782146,-96.48652349968441 32.35063057987183,-96.44018287413185 32.347820408563926,-96.42229948324473 32.32727372554601,-96.40751242042502 32.28861420393443,-96.34356011722869 32.2183314157035,-96.29868578403948 32.21190521188863,-96.27439839885919 32.20550944288857,-96.27315175107162 32.18249126296732,-96.34185031975932 32.14776293117896,-96.38247385130276 32.08753329049883,-96.37545194465046 32.05754749094325,-96.35620789084462 32.03551094988144,-96.3158595907485 32.019917905293234,-96.32056306488315 31.978597683490968,-96.32900004288632 31.969098418198424,-96.34917204756985 31.970915587007834,-96.3551181694795 31.95196052895504,-96.39049861897355 31.953650646994827,-96.39912559502365 31.947157871544896,-96.40129867575553 31.920339760126662,-96.37870928674684 31.885113113233746,-96.3687952578819 31.847274746391868,-96.35539919357339 31.8378431925379,-96.28357581201504 31.821994522914398,-96.22374687914404 31.8297973335714,-96.19883368005854 31.817655531073132,-96.15346611184542 31.82462554016264,-96.11583085915622 31.81570436693316,-96.0847725732137 31.7578069034426,-96.12514532554285 31.716884569650556,-96.1274324876885 31.69567576682241,-96.11192489573175 31.683385086281245,-96.07586080869156 31.6695019335959,-96.05631440774656 31.637593406519635,-96.05101713029102 31.624582283370152,-96.05773810622438 31.612643862824143,-96.1028021755331 31.572031265849194,-96.08762710650876 31.534369883000547,-96.04677144548735 31.512112637175957,-96.00484362737308 31.520167409224168,-95.97913784369514 31.509189004870994,-95.95601472301809 31.526444158863665,-95.94827103265314 31.483833948685863,-95.95485229714987 31.468873503096845,-95.93328154115984 31.470509677586403,-95.91131534636722 31.482481357251952,-95.87762874776206 31.46861825627925,-95.8673946140717 31.44922766711495,-95.88332149323351 31.409105740615118,-95.89991251359915 31.401144621320416,-95.9281225147065 31.409656836767773,-95.95175311916704 31.406489384958064,-95.99791734799405 31.347600934568536,-95.99434778215266 31.307582272250364,-95.97174064740587 31.282368682083934,-95.9496786617975 31.270562493150393,-95.93541352332328 31.273644033617142,-95.8806408732495 31.321930771522126,-95.84107273668609 31.309297532914787,-95.84749556264919 31.337168927881283,-95.83839680886155 31.34895849585119,-95.82577124091462 31.34578725004751,-95.80936677918584 31.30587135093755,-95.78717204285196 31.30038420396976,-95.75193457905735 31.30709681544398,-95.69200329242878 31.354721122145456,-95.68475991847754 31.337481317592424,-95.65374847673156 31.324686504427145,-95.63690854605693 31.330267099966267,-95.62622515770946 31.302390750136972,-95.63818449067686 31.274691802526146,-95.67147906596902 31.243739402094462,-95.67909324426644 31.216754800398157,-95.67079212956742 31.203451316327524,-95.63836528966783 31.194204449569803,-95.62877318321651 31.182483995844045,-95.62953240154835 31.124828249086068,-95.60246123060092 31.102831870057894,-95.57763667204838 31.10111611844306,-95.51705370701802 31.13083855314834,-95.50111299886704 31.13006655160223,-95.46109991532327 31.066125991314262,-95.5106566588292 31.06153565450961,-95.51951598087686 31.03931044789171,-95.50855268720417 31.002254433952164,-95.48221034343781 30.962755495942243,-95.49995979466144 30.937126181315936,-95.49970088381518 30.92164705948581,-95.44464311444446 30.88577404837837,-95.45404564357467 30.821200443341624,-95.46911152174879 30.781926157791258,-95.45117688285876 30.759429020430797,-95.4298526418511 30.75821796297191,-95.37999516508218 30.784874217992908,-95.3619978376101 30.785576105223353,-95.33233547066969 30.73221736441853,-95.31336088822626 30.717343778859416,-95.29148859295017 30.713368406452638,-95.26086752008486 30.728952732887517,-95.22783342750313 30.73188835964981,-95.23350817557153 30.742761028438427,-95.26499847203827 30.74846859048375,-95.26011779716232 30.75517955930153,-95.23900659546212 30.755637421285606,-95.21943706743525 30.74744423408059,-95.19824720720521 30.717884874463905,-95.17253869459363 30.717306082148333,-95.14593520482454 30.687316805047132,-95.11849532406245 30.684234502293734,-95.12035552012497 30.667842014117138,-95.10072883230843 30.669053530494534,-95.08675918049879 30.681158835290983,-95.07929870406963 30.677219876774192,-95.08332579593393 30.654235434307964,-95.06441999661693 30.64739794808772,-95.03170414512364 30.65502673873128,-95.02443528923955 30.649068527573395,-95.00959402964894 30.55910243538991,-94.99388016780453 30.547410840224334,-94.95404269686826 30.54330573351014,-94.94751175276143 30.516768846513823,-94.9201838851466 30.533565485182685,-94.9049908927613 30.541507198767416,-94.88679509368106 30.53974586213126,-94.86241578257803 30.503673703650083,-94.79513321548228 30.479405407010457,-94.78111163192185 30.466441699300763,-94.76778686905335 30.47926159620704,-94.75488233134226 30.479249651500943,-94.73304280896978 30.44567613437868,-94.75026716678398 30.412509435400025,-94.72982684956402 30.3973182428388,-94.74249613050512 30.395627935317957,-94.76069583367007 30.405558259444923,-94.76491986533055 30.3981060497991,-94.73317915831896 30.386747177828482,-94.73785173057192 30.36646231189399,-94.71284141134561 30.35021068417084,-94.71995330456718 30.325031635165477,-94.63296215147062 30.25113956054529,-94.61396087844794 30.273658238636227,-94.60001659678257 30.244350819202005,-94.58924341217892 30.234639110608164,-94.55560065906612 30.250855651363533,-94.56412677689279 30.227806979075375,-94.54917056123 30.224870003820183,-94.5215374742834 30.247349992237552,-94.4846576869792 30.256506781707266,-94.44543075826562 30.18665220817318,-94.43400906278806 30.171756199453483,-94.45415165849835 30.15408855542984,-94.4534072569788 30.14587388672869,-94.42885039055669 30.137341745534023,-94.42288444600163 30.126643604633628,-94.44700921406603 30.116106865010067,-94.45035181953973 30.106198988045076,-94.41085784187416 30.114063758616794,-94.42918929206668 30.09896661562761,-94.42628777750727 30.07730366756627,-94.43848988401552 30.061006289726333,-94.41872517552537 30.04776494375278,-94.4149836954654 30.02296252943233,-94.39215157046253 30.01692792625334,-94.40224471090157 29.998411999696017,-94.39509027129355 29.98044838321186,-94.41757415777481 29.962999313453437,-94.40220418025093 29.949089047527607,-94.40688634808916 29.933847753830666,-94.42278063108066 29.927398567168385,-94.41386487218922 29.896352561924143,-94.42043068216161 29.86290996459473,-94.39410550239187 29.838158221766644,-94.39198455744106 29.820960688944346,-94.36652461413716 29.818049700509324,-94.35378879054055 29.805357756047183,-94.37524062130373 29.809283898274813,-94.38361078087198 29.80389093319544,-94.35828810024327 29.78464765117217,-94.38466227036747 29.77879273036015,-94.36870204982915 29.76717822259514,-94.38705207799516 29.767090715318044,-94.38410419543432 29.754240411275678,-94.35945365261283 29.713928645313235,-94.33865890013787 29.703915373021534,-94.33361163630235 29.69256572106108,-94.33898370033383 29.67499008329111,-94.36047130198997 29.685752551470213,-94.36623773828774 29.682858401900706,-94.36090840206818 29.674217615156465,-94.34187736918854 29.666051675792932,-94.324929232942 29.67135542055625,-94.32021080103739 29.6385693261601,-94.29765849814429 29.637916582270176,-94.30070178190456 29.62762060906107,-94.2873711166681 29.61722492885356,-94.29827329716679 29.613771448041117,-94.31257752297319 29.62595859330978,-94.32066808662921 29.62327583785515,-94.31549047672146 29.6132737473719,-94.28608570595104 29.600068211893245,-94.29553233436039 29.588307945500762,-94.31405395621881 29.597471033792743,-94.32046298970057 29.59225035698941,-94.31486707296912 29.582546702803956,-94.28392633080442 29.569195989919397,-94.30812881085181 29.5580838655309,-94.28965914513664 29.55573586185046,-94.2897673897976 29.5438146984992,-94.26553814987042 29.534658089969447,-94.23407734741107 29.511762662832428,-94.25166534835226 29.498072938838913,-94.27301455563946 29.498960633618882,-94.27436079198552 29.4880868419651,-94.26372267184213 29.47898186526292,-94.26975028633468 29.448049051859,-94.25953337229625 29.440635412244433,-94.23362741159929 29.444515285419495,-94.23077441686098 29.436716162425054,-94.24724806343973 29.425415350705848,-94.2737153256888 29.431385356629168,-94.26823196082236 29.406626199270683,-94.21730118040405 29.397718889294037,-94.21371686234642 29.369198599001383,-94.191081592728 29.383238162982437,-94.16926837729696 29.37722443843496,-94.16875817489036 29.36940897102332,-94.19060604123295 29.35469512882395,-94.1382386764716 29.35254443569014,-94.1345269637809 29.338621470552862,-94.16422348800151 29.329318672072272,-94.15075461439501 29.311016972605625,-94.12309729415884 29.326767904198597,-94.08435410891786 29.30344380463896,-94.10495435628512 29.2816039935465,-94.06145553657181 29.249427480722787,-94.0436219706734 29.24948067119482,-94.03246949176163 29.228407609225325,-94.03396975509612 29.19675498910965,-94.28586351845063 28.993401380809487,-94.50427221366283 28.818837193167557,-94.62967564521237 28.720350918126684,-94.79270490825594 28.586035539270267,-95.05581069289742 28.3656549794488,-95.22052629774547 28.230635499650692,-95.3985553233781 28.085475697864165,-95.38614824882431 28.06524678321548,-95.39212370093259 28.053729199626115,-95.40123206339801 28.013462649751027,-95.41625172406799 27.992444278080022,-95.40489858290964 27.9603688322398,-95.39261298456277 27.948920131450688,-95.4111824501088 27.940402651116855,-95.39316282915567 27.925923282694473,-95.40970945475208 27.904227543974574,-95.40090285475733 27.897833768173058,-95.4167262664833 27.880200391397068,-95.43661204574875 27.848691032279028,-95.42913386643261 27.817945151648818,-95.4444998962919 27.790511872848104,-95.4691294287637 27.781312081903664,-95.47459551528877 27.751209181870724,-95.49351380714424 27.741687755821573,-95.49345946126222 27.73425210826706,-95.47896221858858 27.726174152648294,-95.52405224380254 27.712866495442103,-95.53834286583489 27.69457696122091,-95.54898002678323 27.695896023384865,-95.57806715925717 27.677900861520527,-95.59141445515772 27.686307730681136,-95.61098997315987 27.66664991450626,-95.61122193434613 27.646718754681395,-95.62285596499831 27.630128927018188,-95.6241008982834 27.592281472641318,-95.61529290838841 27.579129617201932,-95.5985741146111 27.57376174217951,-95.60146671965234 27.552127159481476,-95.58505968751719 27.534253124632638,-95.61570654625939 27.525062375684833,-95.64120013083463 27.5422305121139,-95.6461078689536 27.539176827818803,-95.65195525792872 27.504703443887617,-95.6310151276321 27.48751471206297,-95.64649815122262 27.479036580327925,-95.64337705591092 27.46349687181205,-95.65851899577774 27.462112606327565,-95.6675527138838 27.4526342367753,-95.67127417591817 27.441136100928503,-95.6620773842753 27.42020986707046,-95.68241206032236 27.399897169361832,-95.66340622376674 27.383711512513518,-95.73111909684779 27.379188995215358,-95.74771750545088 27.37104470127698,-95.75283174393657 27.335908385729393,-95.74259971242603 27.331535934861925,-95.74666185650418 27.31025197953002,-95.7352808904089 27.30216103888613,-95.73714138283643 27.295064518109506,-95.76049073303311 27.274757968141735,-95.75737089697536 27.25246381039416,-95.79208013248243 27.238898576099228,-95.80220431607128 27.21963150321267,-95.79760362046392 27.202749995210493,-95.76359455929071 27.198743325360045,-95.75333396113886 27.18084670236932,-95.75859844809045 27.174082055109565,-95.7669279741743 27.179482523484168,-95.77254598960894 27.164275302047297,-95.78654381964877 27.17202700668666,-95.80496701966541 27.148696542853504,-95.80417955303076 27.137888163276795,-95.81738380739183 27.1355087010197,-95.83779932360271 27.14494068028703,-95.84759971258237 27.1408742243081,-95.84113470204136 27.12568713391629,-95.85242753492963 27.117566680124913,-95.84065972981287 27.094277729477252,-95.86176986011783 27.087498243090657,-95.86471653404425 27.062506526914685,-95.9130144623978 27.071223326879824,-95.9217332002838 27.08572754277074,-95.92814307130939 27.084031740193392,-95.94356956411886 27.074892031692684,-95.94164628365722 27.064428467808057,-95.96274773808844 27.06101702133803,-95.95175856381196 27.04112713996693,-95.95547839455872 27.02424268710203,-95.94868830722486 27.023576829515388,-95.96558793125058 27.0016139362523,-95.9761298785977 26.998222503698177,-95.99123134182386 27.009676367066014,-96.00587727060291 26.993791862700945,-96.02096812773308 27.003210245999057,-96.0371416500268 26.994748197333283,-96.04424733253761 26.979550242540707,-96.04872725407675 26.966047227203173,-96.0709349483543 26.96432227185933,-96.06601330928287 26.95522072279047,-96.07463506006577 26.94204879091095,-96.11829452877427 26.943986447529046,-96.14608165366307 26.925044960139253,-96.1912227639561 26.92830653234271,-96.19568315808478 26.911445115441484,-96.20209698204499 26.91850684525555,-96.21744953668141 26.899252747063002,-96.24108529551127 26.886044915081875,-96.28173569522174 26.90412288495455,-96.3222693737335 26.892538831528235,-96.33207834316094 26.90261090338369,-96.34065101513788 26.8854065037049,-96.35266683235896 26.885362177027236,-96.35369610512504 26.858755916775618,-96.36764296614635 26.873859374244415,-96.39396292434043 26.88184726682669,-96.41042466223419 26.868314996465386,-96.44156193992738 26.863820267816553,-96.44066544160182 26.828142010249202,-96.46202814955836 26.821667023340087,-96.4646062483347 26.810546416123263,-96.47623699356188 26.810168903522005,-96.48064446215669 26.78591911058343,-96.49076337225773 26.784194803169587,-96.5030701276092 26.76564046238123,-96.52822503472918 26.775298963394857,-96.56307259872668 26.77212481261573,-96.59450779307846 26.761221327026973,-96.60193661403102 26.74638601545648,-96.62103826583458 26.74462078356977,-96.63223585205989 26.735152854390684,-96.64292083109585 26.690379547788304,-96.62860948256724 26.664886292589703,-96.68351224184825 26.629345249292857,-96.70480795258366 26.62084433456158,-96.7312501646986 26.586101984426673,-96.73010392000207 26.577030176326886,-96.75361582596547 26.56180508646482,-96.74460767301314 26.549069665111084,-96.75244867320284 26.546009765600985,-96.76106825676798 26.554373636569057,-96.77970483029938 26.536813713608247,-96.7901843817616 26.542142665326406,-96.81145774240528 26.5336427330796,-96.8390252563819 26.506635003611475,-96.91770247703849 26.499757944167495,-96.9965119570661 26.553168227544422,-97.01797883026167 26.534203568238414,-97.20667527435722 26.49443547503019,-97.24922779309044 26.31600389366909,-97.40588073089162 26.468219138604592,-97.70011140798113 26.580937017198124,-97.72038018471154 26.58860633577142,-98.02806016326744 26.707075068106473,-98.14317380884316 26.714389141200332,-98.13959853975516 26.740193108854694,-97.9920208589544 26.736186720178782,-97.8690336309843 26.704619138714328,-97.8337381413428 26.646591050359913,-97.77922282005328 26.653280750526896,-97.81686038816004 26.672224352869723,-97.82491044039607 26.693524796301237,-97.82460924481236 26.731328519912616,-98.01041207110893 26.850581723936823,-97.85363109726838 26.88062841061066,-97.76817636590583 26.9437926920885,-97.75967192447027 26.994026482537482,-97.85124730755551 27.032533653467542,-97.96768798879161 27.00670915224295,-97.98691976499738 27.0461664697361,-97.98936907935175 27.233611041491926,-98.03515900508417 27.137613776084727,-98.02751443700518 27.081759090776075,-98.14615904793511 27.013975292331953,-98.12613844982351 26.9050036250928,-98.20833224023872 26.924430752867707,-98.17834623768869 26.901594491819996,-98.21842229976663 26.87912705913548,-98.19541739312325 26.850497932855625,-98.17443386077873 26.8721828652778,-98.19783511900276 26.82127230148737,-98.17040908408475 26.75144961762846,-98.20269750961278 26.817658807317382,-98.2793896806476 26.76311190923432,-98.30042244662916 26.80961206877422,-98.48812405669443 26.79493677018952,-98.54845941434486 26.859924705857043,-98.62077761545255 26.805285339291416,-98.65322359161219 26.81818094060506,-98.79448349667534 26.763600285904566,-99.13013122018597 26.825395388516196,-99.30017087035917 26.875627348739027,-99.27066165009136 26.885746651783794,-99.3602317293597 26.951976320514095,-99.50315588331836 27.01785799266927,-99.57270702218668 26.98335235710877,-99.31981114175626 26.882707070122844,-99.8548782606075 27.05539677263233,-99.6187538633943 26.988570630043938,-99.56590911824148 27.02260388666674,-99.80650823661675 27.133075338023684,-99.78589223649814 27.15184047785077,-99.71878027323604 27.107952067031174,-99.7552545481841 27.178850635437747,-99.58753595041121 27.208427221552917,-99.69263378562319 27.196090986055882,-99.76361053438907 27.22732439246448,-99.7134394801508 27.24972153233014,-99.81400196941483 27.234645223031453,-99.85325587681633 27.24849349990996,-99.82819768825308 27.302714952099304,-99.78210227348535 27.341845253014228,-99.81714125384705 27.35472114998323,-99.79639111954911 27.399240623239688,-99.83302272364773 27.3479259173374,-99.79481048322296 27.344335885934477,-99.87296903259488 27.298649227493968,-99.86748417956247 27.250428705609735,-99.99048478603078 27.28983325942198,-99.92032131354264 27.281960393584807,-99.88263902084647 27.333939342162115,-99.95069295592992 27.31297618683615,-99.96162153887397 27.338753146019588,-99.92687403538213 27.367174092990076,-99.97776402872064 27.396513535800494,-99.93131542304569 27.434938459668878,-99.83489802719589 27.51625922200802,-99.93629976868951 27.4380320673504,-99.92412631248979 27.466268901129116,-99.96412264227139 27.50011812224409,-100.0026594115636 27.484795317416133,-100.01296036501137 27.41162574520012,-100.06958723894518 27.369427063596053,-100.02731371954007 27.351232335653084,-100.02978995270973 27.251820689849538,-100.11233837771407 27.276922851859872,-100.08939634450508 27.241497716843355,-100.02861799815305 27.240531987029495,-100.0251467482211 27.139047708062996,-100.31535864199854 27.228865757104792,-100.31485498383222 27.279193736542485,-100.26931554648851 27.318477092784395,-100.28804205681125 27.347950818601838,-100.25754576972035 27.4293103762069,-100.30578357729654 27.434259484101737,-100.28452063958731 27.40861956201743,-100.2942424510174 27.36511423527359,-100.32137833872581 27.362515953515555,-100.36317984838642 27.42297694977367,-100.36282047995195 27.344021810487586,-100.39457151371796 27.31548612159038,-100.43399539176463 27.29159075513323,-100.45861972415382 27.25257227783975,-100.49101697835775 27.257183351886656,-100.67178133355164 27.287165219001412,-100.52709800394267 27.3599515363942,-100.6079477081327 27.369389273392244,-100.60474021876338 27.330447137116487,-100.68761225556572 27.303532217406918,-100.70864929577563 27.357290351054026,-100.64980560925643 27.393536723484864,-100.78105170608289 27.41695491804517,-100.7573133936261 27.440833501020492,-100.78135963557769 27.46058543295877,-100.80985858500748 27.423865719901467,-100.92288470234244 27.45614704208147,-100.92335625557467 27.432638518309556,-100.96198075671865 27.436126732325707,-100.92672673079453 27.423771216094273,-100.83857430630448 27.35896291733622,-100.72847253719665 27.332205699810892,-100.79961079614347 27.27043337670182,-100.94845073306985 27.25816883311316,-101.08904026797116 27.24057058412417,-101.11560263562072 27.28135950250936,-101.08545879991567 27.265256278043864,-101.09972920731853 27.338741649124874,-101.17824623185697 27.36829367065059,-101.14016350691153 27.380914073915697,-101.24764159165173 27.454807589348697,-101.24067355570594 27.481563421550966,-101.26748218456612 27.489863769839534,-101.26940310416725 27.461392641512862,-101.27511851650367 27.432754411674885,-101.20397594905937 27.380501442180638,-101.25976122277108 27.344304819560694,-101.25504361709811 27.252467922424728,-101.27396880084518 27.275425140743174,-101.27447470890812 27.239149637229016,-101.33844490849845 27.226963033409653,-101.3758495939898 27.251882421877102,-101.34224005384331 27.216708553787228,-101.25284647026959 27.243843407213696,-101.22836391176395 27.19176684663845,-101.38869269345335 27.158877342874394,-101.64333874540728 27.047068217472287,-101.70423913949146 27.104161797111065,-101.61009084491027 27.19048393990478,-101.70296809078991 27.139227820993636,-101.78797736385093 27.156385008317113,-101.79071427727817 27.336464155857062,-101.84690999481623 27.20497647045835,-101.92342076305353 27.269257783197858,-101.81058132896072 27.074483310427542,-101.69970895243442 27.01872856442682,-101.90312873851764 26.943619605012714,-101.95669212483017 26.869443149885505,-102.0328017613495 26.915756096886494,-102.13433097023062 26.818951050485058,-102.13502906093976 26.74209039546131,-102.26391639105145 26.5816583336306,-102.31441138691397 26.508156316379267,-102.38155209962073 26.512234044767503,-102.42058076897496 26.414304775601927,-102.40632589699942 26.36183429117501,-102.540708469486 26.23131600371142,-102.56619221582021 26.087041679991152,-102.6359505692475 26.0574415987532,-102.5875307068036 26.020085809171075,-102.60438737136312 25.95578528207801,-102.6872278346894 26.052598628278673,-102.7070245524669 26.0327771483238,-102.77661439540431 26.06887128724015,-102.83831790972867 26.0231835839379,-102.87147296796667 26.05541585413357,-102.90512765875098 26.218629881310036,-102.8777363896967 26.222824262644842,-102.89708876150672 26.253331140972122,-102.87060193431486 26.318332980726822,-102.98936573312731 26.488543273503456,-103.1070191927319 26.602174902888326,-103.15187963055638 26.60934465628763,-103.13106259283677 26.634997054971816,-103.21920367898102 26.705540931184853,-103.24610306034513 26.794718214693507,-103.21348565430402 26.796576963481954,-103.21791347293855 26.8184479951738,-103.25941551770522 26.82762983746796,-103.23396011773661 26.87218606014329,-103.261438095871 26.894998277097805,-103.25498182955346 26.98430528930277,-103.29466434694403 26.99432605178878,-103.32194816269316 27.10754404417207,-103.38987454589889 27.146368728578338,-103.37701202497685 27.32953557733973,-103.44949029759377 27.37803421817408,-103.4413803905899 27.421588040627533,-103.5350317824092 27.547721626730564,-103.47497119808212 27.590711844405828,-103.47338955410855 27.67343782450976,-103.44337579146266 27.694615036132316,-103.2961823654064 27.96007280416516,-103.2937822556293 28.08878450792583,-103.25492250664283 28.123213779983576,-103.26988324623883 28.18615670141249,-103.11689462060701 28.289541704953542,-103.07751579276777 28.355522247277595,-103.09027039040669 28.43932768228092,-102.91177224237072 28.520968573951187,-102.92513492412334 28.55362907684929,-102.8505178375089 28.64131444926327,-102.9219887754126 28.78145303553535,-102.89217892115286 28.92689879006419,-102.91757634421853 28.97689723527919,-102.8057275866267 29.14775522095922,-102.82680896308418 29.192122753014257,-102.77139871374126 29.360508430112013,-102.81108294428981 29.467994993263805,-102.78447858647743 29.500641720905364,-102.80249433976687 29.57468333244619,-102.7656852593807 29.59837730673123,-102.74380144086595 29.685738639170932,-102.69027415531231 29.75875145524409,-102.65074479207804 29.75380364968705,-102.66443885557777 29.830273883920523,-102.61523057926863 29.850439911259556,-102.61978518527107 29.95906101291899,-102.48646989570173 30.17472537596299,-102.50178849441401 30.230318611696585,-102.38383378135255 30.3501628158351,-102.37873391719819 30.47195645782168,-102.33403065851205 30.541086277489875,-102.36758138105 30.763750783328227,-102.32246311723947 30.8681206913329,-102.40933924743575 31.026605192129754,-102.3182518572574 31.09091082531988,-102.39554234751594 31.09318609268347,-102.33298747578623 31.13923289543434,-102.3713850351829 31.181939698608776,-102.32902103965627 31.27127602049616,-102.29825001234047 31.278330602408545,-102.33810698511643 31.30184361118353,-102.32924879965309 31.334599390077738,-102.39701047641508 31.360211716132042,-102.36091736777473 31.39531628488143,-102.42445601378654 31.388851251555838,-102.47083598477494 31.420047404764652,-102.52621947781171 31.51551241200745,-102.56355301141663 31.539750537124036,-102.54406444217885 31.571325424222657,-102.63704224237154 31.62033400612857,-102.64468332132041 31.671439290663766,-102.7351313510915 31.70558413006314,-102.83066897072283 31.932743647816363,-102.88949693605392 31.936623559687394,-102.96670396991156 31.895886283239367,-103.03207951021669 31.98170432988653,-103.09564766158091 31.988375529137198,-103.0879485609851 32.02288495971939,-103.16825325153717 32.03061136320936,-103.18446113093985 32.0627772621078,-103.45254287771684 32.00091175983358,-103.56428089762824 31.933331427957985,-103.65483390178474 31.932651575207338,-103.74295139151226 31.8927041343779,-103.74821472330981 31.839625083358467,-103.86302473154605 31.89080898598726,-104.153616722605 31.854073208920042,-104.20612875115648 31.944037034715127,-104.23317687739518 31.93873150744154,-104.21680879887205 32.015568195745324,-104.25921013659382 32.028531491296135,-104.29241708680611 32.116692991853405,-104.26508968984477 32.198496414933736,-104.35963818249488 32.34871482006478,-104.33943571287675 32.387086161085236,-104.36858380758248 32.413103851503436,-104.31547754770448 32.43803397261113,-104.34029874412302 32.440809566944814,-104.46434705789576 32.641923248353336,-104.48362523081467 32.763477365872006,-104.44064647143473 32.84331298069573,-104.50160550811836 32.994227067269314,-104.49953651750825 33.17637610182482,-104.41945578086624 33.28681946740748,-104.39918755743693 33.41303497249706,-104.28856859156052 33.523764898317275,-104.19594752678948 33.570533192325804,-104.13869347709458 33.64642251904792,-104.11884562389291 33.78829494389068,-104.09163980918083 33.80074378487681,-104.10970515118234 33.839662090725895,-103.9831706199966 33.97970232456346,-103.98798601375712 34.085018929694726,-103.95842617326018 34.11970651669095,-103.99812852540725 34.16054249166227,-103.99613716081028 34.33578492705304,-104.03821562155852 34.35306929031097,-104.0288012286996 34.393444459541975,-104.0597716225304 34.39929366758984,-104.08034540627634 34.46983146447429,-104.0507240589672 34.513573798858246,-104.06715203024667 34.66452897142356,-104.02463776836149 34.751771121242186,-104.06509088944316 34.90778761839033,-104.02825076334733 35.19149566106441)),((-98.33246913768639 26.737042952567876,-98.16190570914539 26.69988962123285,-98.16840161125498 26.671320369425985,-98.59785333128735 26.751777560637755,-98.33246913768639 26.737042952567876)),((-100.10370850341465 27.08292084267071,-100.63297035140751 27.17776280875662,-100.52388648334697 27.23027372719067,-100.47037827577107 27.195419764515307,-100.2177788295244 27.152206468798223,-100.1646065603277 27.11199872171557,-100.08234679577463 27.130589116778033,-100.10370850341465 27.08292084267071)),((-100.7213703779688 27.23299187435441,-100.60436061103673 27.247322758126273,-100.6052744553987 27.204010747741556,-100.66006865029534 27.17874732673494,-100.96626859470034 27.16439795622006,-100.88813746439453 27.198211052094948,-100.77889004679082 27.197024132802916,-100.7813069011771 27.223750924611863,-100.7213703779688 27.23299187435441)),((-101.6381687943433 26.986585957325158,-101.71248467597178 26.949870924128092,-101.64891221995633 26.998396373620587,-101.49212807973989 27.078437411805293,-101.31696553830274 27.117304544446835,-101.26516837258123 27.172223414305954,-101.26074067613472 27.108819270939588,-101.15892252670953 27.152183087033357,-101.00835440375523 27.1609363909816,-101.02734217551192 27.191194232271954,-100.97766597952322 27.15981420070737,-101.30595654193877 27.096010536208894,-101.6381687943433 26.986585957325158)),((-102.1618062361393 26.48212713745765,-102.11350339938745 26.5918125110623,-102.0356730451307 26.6848135600667,-101.96141270573742 26.763389832662597,-101.80906161174391 26.896282193605654,-101.72613745790665 26.950054831997253,-101.71687299979766 26.945211146486766,-101.81112778088483 26.87751557979775,-101.92964613106795 26.772621080459803,-102.0344590720666 26.65733452129484,-102.15850660245347 26.478383524492475,-102.26524038822303 26.311654616063592,-102.37041381610986 26.178649983898037,-102.50336238481229 26.039461982561132,-102.51322036505677 26.04051222629201,-102.38698398346936 26.181951723995127,-102.31841314621954 26.260336945082294,-102.27726426637639 26.325128134346024,-102.23917696745114 26.3763843869855,-102.19668068048563 26.425522222544124,-102.1618062361393 26.48212713745765)))");

        request.setAttributes(Arrays.asList(geometry, attr, attr1));

        logger.info("\n\nHERE THE REQUEST ########################## \n {}",
                request.showRequestAsString());

        WFSGetFeatureRequest<FeatureCollectionType> requestGet =
                super.serverConnector.createGetFeatureRequest();

        requestGet.setResultType(ResultTypeType.RESULTS.value());
        requestGet.setTypeName(statesName);

        requestGet.setFeatureIDs(Arrays.asList("states.15"));

        logger.info("\n\n\nRESPONSE GET FEATURE @@@@@@@@@@@@@@@@@@@@@@@@@@@@@ "
                + "{}", requestGet.getResponseAsString());

    }

    @Test
    public void sfRoads() throws Exception {
        WFSTransactionRequest<TransactionResponseType> request =
                super.serverConnector.createTransactionRequest();

        request.setOperation(TransactionOperation.UPDATE);
        request.setTypeName(sfRoads);

        request.setFID("roads.23");

        GeometryAttributeDTO geometry = new GeometryAttributeDTO();
        geometry.setName("the_geom");
        geometry.setSrid(new Integer(4326));

        geometry.setValue(
                "MULTILINESTRING((-103.89135254479997 44.62794736744623,-103.89096632362241 44.627735453174864,-103.89062170872486 44.62730973343466,-103.89002953848615 44.625163198149565,-103.89024101107378 44.62483551115963,-103.89114993132819 44.62388221415969,-103.89139774483179 44.623670144891136))");

        request.setAttributes(Arrays.asList(geometry));

        logger.info("\n\nHERE THE REQUEST SF_ROADS ########################## "
                + "\n {}", request.showRequestAsString());

        WFSGetFeatureRequest<FeatureCollectionType> requestGet =
                super.serverConnector.createGetFeatureRequest();

        requestGet.setResultType(ResultTypeType.RESULTS.value());
        requestGet.setTypeName(sfRoads);

        requestGet.setFeatureIDs(Arrays.asList("roads.23"));
        
        logger.info("\n\nHERE THE REQUEST GET_FEATURE SF_ROADS ########################## "
                + "\n {}", requestGet.showRequestAsString());

        logger.info("\n\n\nRESPONSE GET FEATURE SF_ROADS "
                + "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ {}",
                requestGet.getResponseAsString());

    }

//    @Test
//    public void testFiresat() throws Exception {
//        WFSTransactionRequest<TransactionResponseType> request =
//                super.serverConnector.createTransactionRequest();
//
//        request.setOperation(TransactionOperation.UPDATE);
//
//        QName name = new QName(FIRESAT.getLocalPart());
//        request.setTypeName(name);
//
//        request.setFID("firesat.10234");
//
//        AttributeDTO att = new AttributeDTO();
//        att.setName("AREA");
//        
//        att.setValue("33");
//        request.setAttributes(Arrays.asList(att));
//
//        // We must avoid transactions on the server,
//        // but show only the requests generated
//
//        logger.info("HERE THE REQUEST ########################## \n {}",
//                request.showRequestAsString());
//        
//        TransactionResponseType response = request.getResponse();
//        logger.info("\n*** {}", response.getTransactionResults());
//
//        TransactionSummaryType transactionSummary = response.getTransactionSummary();
//        Assert.assertEquals(0, transactionSummary.getTotalDeleted().intValue());
//        Assert.assertEquals(0, transactionSummary.getTotalInserted().intValue());
//        Assert.assertEquals(1, transactionSummary.getTotalUpdated().intValue());
//        Assert.assertEquals("1.1.0", response.getVersion());
//    }
}
